/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.AppliPanel;

import com.google.gwt.dom.client.Style.Unit;
import com.google.gwt.event.logical.shared.ResizeEvent;
import com.google.gwt.event.logical.shared.ResizeHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Widget;
import org.ow2.orchestra.console.GWTConsole.client.Commons.PresenterIF;
import org.ow2.orchestra.console.GWTConsole.client.ConsoleEventBus;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneHandler;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;


/**
 * Class that contains the presenter of the application panel
 *
 * @author Mathieu Crassin
 *
 */
public class PAppliPanel extends PresenterIF {

  private VAppliPanelIF view;


  private PTabLayoutPanel pTabLayoutPanel = null;

  /**
   * Constructor of appli panel with tab layout panel
   *
   * @author Mathieu Crassin
   * @param eventBus : the handler manager shared
   * @param view : Complete class view of our application
   * @param pTabLayoutPanel : the presenter of tab layout panel
   *
   */
  public PAppliPanel(final VAppliPanelIF view, final PTabLayoutPanel pTabLayoutPanel) {
    //
    this.bindingView(view);
    //
    this.addHandlerToBus();
    this.pTabLayoutPanel  = pTabLayoutPanel;
    //
    this.addListeners();
    //
    this.view.getAppliLayoutPanelChild().add((Widget)pTabLayoutPanel.getView());
    //
    this.view.getAppliLayoutPanelChild().setWidgetTopBottom(
        this.view.getAppliLayoutPanelChild().getWidget(1),
        120, Unit.PX, 0, Unit.PX);
    //
    this.adjustSize();

  }



  /**
   * This method add handler to the handler manager to adjust appli size when refresh event is fired
   *
   * @author Jean-Yves Goubet
   *
   */
  private void addHandlerToBus() {
    ConsoleEventBus.getEventBus().addHandler(RefreshDoneEvent.TYPE, new RefreshDoneHandler() {
      public void onRefreshDone(final RefreshDoneEvent refreshDoneEvent) {
        PAppliPanel.this.adjustSize();
      }
    });
  }


  /**
   * Method to binf the view with its presenter
   *
   * @author Jean-Yves Goubet
   * @param view : the view to bind
   *
   */
  @Override
  protected void bindingView(final Object viewToBind) {
    this.view = (VAppliPanelIF) viewToBind;
  }


  /**
   * Method to add all listeners
   *
   * @author Jean-Yves Goubet
   *
   */
  @Override
  protected void addListeners() {
    this.addWindowListeners();
  }

  /**
   * Method to add listeners on window resize
   *
   * @author Jean-Yves Goubet
   *
   */
  private void addWindowListeners() {
    final String res = Window.Location.getParameter("resize");
    if (res != null && res.equals("false")) {
      System.out.println("no RESIZE");
    } else {
      System.out.println("RESIZE");

      Window.addResizeHandler(new ResizeHandler() {

        public void onResize(final ResizeEvent arg0) {
          PAppliPanel.this.adjustSize();
        }
      });
    }
  }


  /**
   * Function that returns a string representing the browser name.
   *
   * @author Mathieu Crassin
   * @return String : the browser name
   */
  public static native String getUserAgent() /*-{
  return navigator.userAgent.toLowerCase();
  }-*/;


  /**
   * This method is used to adjust the width and height of appli depending on
   * the client's size window and the tab layout content size too.
   *
   * @author Jean-Yves Goubet
   *
   */
  private void adjustSize() {
    // check if the resize parameter is set or not
    final String res = Window.Location.getParameter("resize");
    // first of all, get the window width
    final int width = Window.getClientWidth();
    // and window height
    final int heightClient = Window.getClientHeight();
    int heightContent = 0;

    // adjust width
    // 924 is the total width needed by the application
    if (width > 924) {
      // in this case, we just add some margin
      this.view.getAppliLayoutPanelParent().setWidgetLeftWidth(this.view.getAppliLayoutPanelChild(),
          (width - 924) / 2, Unit.PX, 924, Unit.PX);
    } else {
      // and in this case not
      this.view.getAppliLayoutPanelParent().setWidgetLeftWidth(this.view.getAppliLayoutPanelChild(),
          0, Unit.PX, 924, Unit.PX);
    }

    // adjust height
    if (Window.getTitle().equals("Login Page")) {
      // min size required for login page content
      heightContent = 400;
    } else {
      heightContent = this.pTabLayoutPanel.getHeightContent();
    }

    if (res != null && res.equals("false")) {
      // just for tests
      this.view.getAppliLayoutPanelParent().setHeight("600px");
    } else if (heightClient < heightContent) {
      // if the height needed by the application is bigger than window height
      this.view.getAppliLayoutPanelParent().setHeight(heightContent + "px");
    } else {
      // otherwise
      this.view.getAppliLayoutPanelParent().setHeight(heightClient + "px");
    }
  }


  /**
   * Method used to set debug ids to item in the view
   *
   * @author Mathieu Crassin
   *
   */
  @Override
  protected void setDebugID() {
    // no debug ID in the appli view
  }

  @Override
  public void destroy() {

  }

}
