/**
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.GWTConsole.client.Commons.celltable;

import com.google.gwt.cell.client.AbstractCell;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.google.gwt.safehtml.shared.SafeHtmlUtils;
import com.google.gwt.user.client.ui.UIObject;


/**
 * @author Guillaume Porcher
 *
 */
public abstract class AbstractDebuggableCell<C> extends AbstractCell<C> {

  private final String debugPrefix;

  public AbstractDebuggableCell(final String debugPrefix) {
    this.debugPrefix = debugPrefix;
  }


  public AbstractDebuggableCell(final String debugPrefix, final String... consumedEvents) {
    super(consumedEvents);
    this.debugPrefix = debugPrefix;
  }

  @Override
  public void render(final Context context, final C value, final SafeHtmlBuilder sb) {
    final UIObject renderObject = this.renderAsUIObject(context, value);
    if (renderObject != null) {
      renderObject.ensureDebugId(this.debugPrefix + "-" + context.getIndex() + "-" + context.getColumn());
      sb.append(SafeHtmlUtils.fromTrustedString(renderObject.toString()));
    }
  }

  /**
   * @param context
   * @param value
   * @return
   */
  protected abstract UIObject renderAsUIObject(Context context, C value);

}
