/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.ListInstance;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import org.ow2.orchestra.console.GWTConsole.client.Commons.PresenterIF;
import org.ow2.orchestra.console.GWTConsole.client.ConsoleEventBus;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshHandler;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;
import org.ow2.orchestra.facade.runtime.ActivityState;

/**
 * Class that contains the presenter part of ListInstance.
 *
 * @author Jean-Yves Goubet
 */
public class PListInstance extends PresenterIF {

  private VListInstanceIF view;

  private final String processId;

  private ViewRefreshHandler refreshHandler = null;
  private HandlerRegistration handlerRegistration;
  private final PListInstanceCellTable pListInstanceCellTable;

  /**
   * The constructor of the class
   *
   * @param pTabLayoutPanel
   * @author Jean-Yves Goubet
   */
  public PListInstance(final VListInstanceIF vListInstance, final PTabLayoutPanel pTabLayoutPanel) {
    this(vListInstance, pTabLayoutPanel, null);
  }

  public PListInstance(final VListInstanceIF vListInstance, final PTabLayoutPanel pTabLayoutPanel, final String processId) {
    this.processId = processId;
    this.bindingView(vListInstance);

    if (this.processId == null) {
      // This is the second tab with instances of all process
      this.pListInstanceCellTable =
              new PListInstanceCellTable(processId, pTabLayoutPanel, vListInstance.getTable(), "instancesCellTable");
      setDebugId("");
    } else {
      // This is the tab with instance of the processID so suffix debug Id with processId
      this.pListInstanceCellTable =
              new PListInstanceCellTable(processId, pTabLayoutPanel, vListInstance.getTable(), "instancesCellTable-" + processId);
      setDebugId("-" + processId);
    }
    this.view.getPager().setDisplay(this.view.getTable());

    this.addListeners();

    // Filter elements

    // State values
    PListInstance.this.view.getState().addItem("ALL");
    for (ActivityState p : ActivityState.values()) {
      PListInstance.this.view.getState().addItem(p.toString());
    }


    this.view.getFilter().addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent event) {
        PListInstance.this.pListInstanceCellTable.dataProvider.setFilterState(
                PListInstance.this.getCurrentSelectedActivityState());
        PListInstance.this.pListInstanceCellTable.dataProvider.setFilterStartFrom(PListInstance.this.view.getStartFrom().getValue());
        PListInstance.this.pListInstanceCellTable.dataProvider.setFilterStartTo(PListInstance.this.view.getStartTo().getValue());
        PListInstance.this.pListInstanceCellTable.dataProvider.setFilterEndFrom(PListInstance.this.view.getEndFrom().getValue());
        PListInstance.this.pListInstanceCellTable.dataProvider.setFilterEndTo(PListInstance.this.view.getEndTo().getValue());
        PListInstance.this.view.getTable().setVisibleRangeAndClearData(
                PListInstance.this.view.getTable().getVisibleRange(), true);
      }
    });

    this.view.getClear().addClickHandler(new ClickHandler() {
      public void onClick(ClickEvent event) {
        PListInstance.this.pListInstanceCellTable.dataProvider.setFilterState(null);
        PListInstance.this.view.getState().setItemSelected(0, true);
        PListInstance.this.pListInstanceCellTable.dataProvider.setFilterStartFrom(null);
        PListInstance.this.view.setStartFrom(null);
        PListInstance.this.pListInstanceCellTable.dataProvider.setFilterStartTo(null);
        PListInstance.this.view.setEndTo(null);
        PListInstance.this.pListInstanceCellTable.dataProvider.setFilterEndFrom(null);
        PListInstance.this.view.setEndFrom(null);
        PListInstance.this.pListInstanceCellTable.dataProvider.setFilterEndTo(null);
        PListInstance.this.view.setStartTo(null);
        PListInstance.this.view.getTable().setVisibleRangeAndClearData(
                PListInstance.this.view.getTable().getVisibleRange(), true);
      }
    });
  }

  /**
   * This constructor is used when adding a tab for a specific process.
   *
   * @param processId : the id of the process selected
   * @author Jean-Yves Goubet
   */
  public PListInstance(final PTabLayoutPanel pTabLayoutPanel, final String processId, final String elementName) {
    this(new VListInstance(), pTabLayoutPanel, processId);
    pTabLayoutPanel.addTab(processId, elementName, PTab.Type.LIST_INSTANCE, (VListInstance) this.view, this);

  }

  @Override
  protected void addListeners() {
    this.addViewRefreshHandlerToBus();
  }

  /**
   * The handler to get events to launch refreshes
   *
   * @author Jean-Yves Goubet
   */
  private void addViewRefreshHandlerToBus() {

    this.refreshHandler = new ViewRefreshHandler() {

      public void onRefresh(final ViewRefreshEvent viewRefreshEvent) {
        if (viewRefreshEvent.getElementType() == PTab.Type.LIST_INSTANCE
                && ((viewRefreshEvent.getElementId() == null && PListInstance.this.processId == null)
                || (viewRefreshEvent.getElementId() != null
                && viewRefreshEvent.getElementId().equals(PListInstance.this.processId)))) {
          PListInstance.this.view.getTable().setVisibleRangeAndClearData(
                  PListInstance.this.view.getTable().getVisibleRange(), true);
        }
      }
    };

    this.handlerRegistration = ConsoleEventBus.getEventBus().addHandler(ViewRefreshEvent.TYPE, this.refreshHandler);
  }

  /**
   * Attach our interface of the view with the real class.
   *
   * @param lp : interface that represents the view of the class
   * @author Jean-Yves Goubet
   */
  @Override
  protected void bindingView(final Object lp) {
    this.view = (VListInstanceIF) lp;
  }

  /**
   * Get method of the view.
   *
   * @author Jean-Yves Goubet
   */
  public VListInstanceIF getView() {
    return this.view;
  }

  @Override
  protected void setDebugID() {

  }

  /**
   * Destroy the ListInstance object. This function remove the handler add on
   * the eventBus in the constructor.
   *
   * @author Mathieu Crassin
   */
  @Override
  public void destroy() {
    if (this.handlerRegistration != null) {
      this.handlerRegistration.removeHandler();
    }
  }

  // return the corresponding ActivityState of the listBox state current selection if possible else return null
  private ActivityState getCurrentSelectedActivityState() {
    int currentSelection = PListInstance.this.view.getState().getSelectedIndex();
    if (currentSelection == 0)
      return null; // case ALL added on first position => no ActivityState corresponding
    else
      return ActivityState.values()[currentSelection - 1];
  }

  private void setDebugId(String processId) {
     this.view.getPager().ensureDebugId("instancesPager" + processId );

       // Filter debug id
     this.view.getState().ensureDebugId("stateListInstanceFilter" + processId);
     this.view.getFilter().ensureDebugId("filterListInstanceFilter" + processId);
     this.view.getClear().ensureDebugId("clearListInstanceFilter" + processId);
     this.view.getStartFrom().ensureDebugId("startFromListInstanceFilter" + processId);
     this.view.getStartTo().ensureDebugId("startToListInstanceFilter" + processId);
     this.view.getEndFrom().ensureDebugId("endFromListInstanceFilter" + processId);
     this.view.getEndTo().ensureDebugId("endToListInstanceFilter" + processId);
     this.view.getImageFilterDisclosure().ensureDebugId("imageDisclosureListInstanceFilter" + processId);
     this.view.getDisclosuredPanel().ensureDebugId("disclosurePanelListInstanceFilter" + processId);
   }
}
