/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.ListInstance;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.logical.shared.CloseEvent;
import com.google.gwt.event.logical.shared.CloseHandler;
import com.google.gwt.event.logical.shared.OpenEvent;
import com.google.gwt.event.logical.shared.OpenHandler;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.resources.client.ClientBundle;
import com.google.gwt.resources.client.ImageResource;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.uibinder.client.UiTemplate;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.DisclosurePanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.Widget;

import com.google.gwt.user.datepicker.client.DateBox;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.ConsoleCellTable;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.ConsolePager;
import org.ow2.orchestra.facade.runtime.ActivityState;

import java.util.Date;


/**
 * This class is the view of ListInstance
 * 
 * @author Jean-Yves Goubet
 * 
 */
public class VListInstance extends Composite implements VListInstanceIF {

  @UiTemplate("VListInstance.ui.xml")
  interface MyUiBinder extends UiBinder<Widget, VListInstance> {
  }

  interface ResourcesFilter extends ClientBundle {
    @Source("close.png")
    ImageResource close();

    @Source("open.png")
    ImageResource open();
  }

  private static MyUiBinder uiBinder = GWT.create(MyUiBinder.class);

  @UiField
  ConsoleCellTable<MInstance> table;

  @UiField
  ConsolePager pager;

  @UiField
  Button filter;

  @UiField
  Button clear;

  @UiField
  ListBox state;

  @UiField
  DateBox startFrom;

  @UiField
  DateBox startTo;

  @UiField
  DateBox endFrom;

  @UiField
  DateBox endTo;

  @UiField
  Image imageFilterDisclosure;

  @UiField
  DisclosurePanel disclosuredPanel;

  /**
   * The constructor of the class
   * 
   * @author Jean-Yves Goubet
   * 
   */
  public VListInstance() {
    //
    this.initWidget(VListInstance.uiBinder.createAndBindUi(this));
    //
    this.initView();
  }

  private void initView() {
    this.startFrom.setFormat(new DateBox.DefaultFormat(DateTimeFormat.getFormat("yyyy-MM-dd HH:mm:ss")));
    this.startTo.setFormat(new DateBox.DefaultFormat(DateTimeFormat.getFormat("yyyy-MM-dd HH:mm:ss")));

    this.endFrom.setFormat(new DateBox.DefaultFormat(DateTimeFormat.getFormat("yyyy-MM-dd HH:mm:ss")));
    this.endTo.setFormat(new DateBox.DefaultFormat(DateTimeFormat.getFormat("yyyy-MM-dd HH:mm:ss")));


    // Add ressources for disclosurePanel
    final ResourcesFilter resources = GWT.create(ResourcesFilter.class);
    imageFilterDisclosure.setResource(resources.close());

    disclosuredPanel.addOpenHandler(new OpenHandler<DisclosurePanel>() {
      public void onOpen(OpenEvent<DisclosurePanel> disclosurePanelOpenEvent) {
        imageFilterDisclosure.setResource(resources.open());
      }
    });

    disclosuredPanel.addCloseHandler(new CloseHandler<DisclosurePanel>() {
      public void onClose(CloseEvent<DisclosurePanel> disclosurePanelCloseEvent) {
        imageFilterDisclosure.setResource(resources.close());
      }
    });
  }

  public ConsoleCellTable<MInstance> getTable() {
    return this.table;
  }

  public ConsolePager getPager() {
    return this.pager;
  }

  public Button getFilter() {
    return filter;
  }

  public Button getClear() {
    return clear;
  }

  public DateBox getStartFrom() {
    return startFrom;
  }

  public void setStartFrom(Date d) {
    startFrom.setValue(d);
  }

  public void setStartTo(Date d) {
    startTo.setValue(d);
  }

  public DateBox getStartTo() {
    return startTo;
  }

  public DateBox getEndFrom() {
    return endFrom;
  }

  public void setEndFrom(Date d) {
    endFrom.setValue(d);
  }

  public void setEndTo(Date d) {
    endTo.setValue(d);
  }

  public DateBox getEndTo() {
    return endTo;
  }

  public Image getImageFilterDisclosure() {
    return imageFilterDisclosure;
  }

  public DisclosurePanel getDisclosuredPanel() {
    return disclosuredPanel;
  }

  public ListBox getState() {
    return state;
  }

}
