/**
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.GWTConsole.client.ListProcess;

import com.google.gwt.cell.client.Cell;
import com.google.gwt.cell.client.CompositeCell;
import com.google.gwt.cell.client.HasCell;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.cellview.client.Column;
import com.google.gwt.user.cellview.client.ColumnSortEvent.ListHandler;
import com.google.gwt.user.cellview.client.IdentityColumn;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.view.client.ListDataProvider;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.ConsoleCellTable;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.ImageActionCell;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.LabelCell;
import org.ow2.orchestra.console.GWTConsole.client.Commons.celltable.NullableDateCell;
import org.ow2.orchestra.console.GWTConsole.client.ConsoleEventBus;
import org.ow2.orchestra.console.GWTConsole.client.Event.RefreshDoneEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.SuspendTimerEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.ListInstance.PListInstance;
import org.ow2.orchestra.console.GWTConsole.client.ListProcessActivity.PListProcessActivity;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementException;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServices;
import org.ow2.orchestra.console.GWTConsole.client.OrchestraManagementServicesAsync;
import org.ow2.orchestra.console.GWTConsole.client.Popup.PPopup;
import org.ow2.orchestra.console.GWTConsole.client.Resources.ImageResources;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;


/**
 * @author Guillaume Porcher
 *
 */
public class PListProcessCellTable {


  protected OrchestraManagementServicesAsync orchestramanagement = GWT.create(OrchestraManagementServices.class);

  // Create a data provider.
  private final ListDataProvider<MProcess> dataProvider = new ListDataProvider<MProcess>();

  private final PTabLayoutPanel tabLayoutPanel;

  private final String processId;

  private final String debugPrefix;

  public PListProcessCellTable(
      final PTabLayoutPanel ptabLayoutPanel, final ConsoleCellTable<MProcess> table, final String debugPrefix) {
    this(null, ptabLayoutPanel, table, debugPrefix);
  }

  public PListProcessCellTable(
      final String processId,
      final PTabLayoutPanel ptabLayoutPanel,
      final ConsoleCellTable<MProcess> table,
      final String debugPrefix) {
    this.processId = processId;
    this.tabLayoutPanel = ptabLayoutPanel;
    this.debugPrefix = debugPrefix;

    table.ensureDebugId(debugPrefix);

    // Add a ColumnSortEvent.ListHandler to connect sorting to the
    // java.util.List.
    final ListHandler<MProcess> columnSortHandler = new ListHandler<MProcess>(this.dataProvider.getList());
    table.addColumnSortHandler(columnSortHandler);

    this.setColumns(table, columnSortHandler);

    // Connect the table to the data provider.
    this.dataProvider.addDataDisplay(table);

    // Fetch data
    this.refresh();

  }

  private void setColumns(final ConsoleCellTable<MProcess> table, final ListHandler<MProcess> columnSortHandler) {
    /* UUID column */
    final Column<MProcess, String> uuidColumn = new Column<MProcess, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MProcess mprocess) {
        return mprocess.getProcessId();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, uuidColumn, "UUID");

    /* Name column */
    final Column<MProcess, String> nameColumn = new Column<MProcess, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MProcess mprocess) {
        return mprocess.getProcessName();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, nameColumn, "Name");

    /* Namespace column */
    final Column<MProcess, String> namespaceColumn = new Column<MProcess, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MProcess mprocess) {
        return mprocess.getProcessNamespace();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, namespaceColumn, "Namespace");

    /* Version column */
    final Column<MProcess, String> versionColumn = new Column<MProcess, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MProcess mprocess) {
        return Integer.toString(mprocess.getProcessVersion());
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, versionColumn, "Version");

    /* Deployment date column */
    final Column<MProcess, Date> deployDateColumn = new Column<MProcess, Date>(new NullableDateCell(this.debugPrefix)) {
      @Override
      public Date getValue(final MProcess mprocess) {
        return mprocess.getProcessDeploymentDate();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, deployDateColumn, "Deployment Date");

    /* Undeployment date column */
    final Column<MProcess, Date> undeployDateColumn = new Column<MProcess, Date>(new NullableDateCell(this.debugPrefix)) {
      @Override
      public Date getValue(final MProcess mprocess) {
        return mprocess.getProcessUndeploymentDate();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, undeployDateColumn, "Undeployment Date");

    /* State column */
    final Column<MProcess, String> stateColumn = new Column<MProcess, String>(new LabelCell(this.debugPrefix)) {
      @Override
      public String getValue(final MProcess mprocess) {
        return mprocess.getProcessState();
      }
    };
    // Add the columns.
    table.addSortedColumn(columnSortHandler, stateColumn, "State");


    /* Actions column */
    final List<HasCell<MProcess, ?>> actionList = new ArrayList<HasCell<MProcess,?>>();
    actionList.add(new IdentityColumn<MProcess>(this.getListInstancesActionCell()));
    actionList.add(new IdentityColumn<MProcess>(this.getProcessDetailsActionCell()));
    actionList.add(new IdentityColumn<MProcess>(this.getRetireActionCell()));
    actionList.add(new IdentityColumn<MProcess>(this.getActivateActionCell()));
    actionList.add(new IdentityColumn<MProcess>(this.getUndeployActionCell()));
    actionList.add(new IdentityColumn<MProcess>(this.getRemoveActionCell()));

    final CompositeCell<MProcess> actionCells = new CompositeCell<MProcess>(actionList);
    final Column<MProcess, MProcess> actionsColumn = new IdentityColumn<MProcess>(actionCells);
    actionsColumn.setSortable(false);
    // Add the columns.
    table.addColumn(actionsColumn, "Actions");
  }

  private Cell<MProcess> getRemoveActionCell() {
    final Cell<MProcess> removeAction =
      new ImageActionCell<MProcess>(ImageResources.INSTANCE.deleteIcon(), ImageResources.INSTANCE.deleteIconDisabled(), "remove", this.debugPrefix + "-remove") {

        @Override
        public void execute(final MProcess object) {
          if (this.isActive(object)) {
            final String processId = object.getProcessId();
            PListProcessCellTable.this.orchestramanagement.removeProcess(processId,
                new AsyncCallback<Void>() {
                  public void onSuccess(final Void arg0) {
                    // delete list instance of a process tab if exists
                    PListProcessCellTable.this.tabLayoutPanel.removeTab(processId, PTab.Type.LIST_INSTANCE);
                    // delete also activity list tab if exists
                    PListProcessCellTable.this.tabLayoutPanel.removeTab(processId, PTab.Type.LIST_PROCESS_ACTIVITY);
                    ConsoleEventBus.getEventBus().fireEvent(new ViewRefreshEvent(processId, PTab.Type.LIST_PROCESS_ACTIVITY, true));
                  }
                  public void onFailure(final Throwable caught) {
                    GWT.log("PListProcess - undeployProcess: on Failure : " + caught.getMessage());
                    if (caught instanceof OrchestraManagementException) {
                      PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                          ((OrchestraManagementException)caught).getDetails());
                    } else {
                      PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while removing process !", caught.getMessage());
                    }
                  }
                });
          }
        }

        @Override
        public boolean isActive(final MProcess value) {
          return value.getProcessState().equals("undeployed");
        }
      };
    return removeAction;
  }

  private Cell<MProcess> getUndeployActionCell() {
    final Cell<MProcess> undeployAction =
      new ImageActionCell<MProcess>(ImageResources.INSTANCE.undeployIcon(), ImageResources.INSTANCE.undeployIconDisabled(), "undeploy", this.debugPrefix + "-undeploy") {
        @Override
        public void execute(final MProcess object) {
          if (this.isActive(object)) {
            final String processId = object.getProcessId();
            PListProcessCellTable.this.orchestramanagement.undeployFromOrchestra(processId,
                new AsyncCallback<Void>() {
                  public void onSuccess(final Void arg0) {
                    ConsoleEventBus.getEventBus().fireEvent(new ViewRefreshEvent(processId, PTab.Type.LIST_PROCESS_ACTIVITY, true));
                  }
                  public void onFailure(final Throwable caught) {
                    GWT.log("PListProcess - undeployProcess: on Failure : " + caught.getMessage());
                    if (caught instanceof OrchestraManagementException) {
                      PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                          ((OrchestraManagementException)caught).getDetails());
                    } else {
                      PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while undeploy file !", caught.getMessage());
                    }
                  }
                });
          }
        }

        @Override
        public boolean isActive(final MProcess value) {
          return !value.getProcessState().equals("undeployed");
        }
      };
    return undeployAction;
  }

  private Cell<MProcess> getActivateActionCell() {
    final Cell<MProcess> activateAction =
      new ImageActionCell<MProcess>(ImageResources.INSTANCE.resume(), ImageResources.INSTANCE.resumeDisabled(), "resume", this.debugPrefix + "-resume") {
        @Override
        public void execute(final MProcess object) {
          if (this.isActive(object)) {
            final String processId = object.getProcessId();
            PListProcessCellTable.this.orchestramanagement.activateProcess(processId,
                new AsyncCallback<Void>() {
                  public void onSuccess(final Void arg0) {
                    ConsoleEventBus.getEventBus().fireEvent(new ViewRefreshEvent(processId, PTab.Type.LIST_PROCESS_ACTIVITY, true));
                  }
                  public void onFailure(final Throwable caught) {
                    GWT.log("PListProcess - activateProcess: on Failure : " + caught.getMessage());
                    if (caught instanceof OrchestraManagementException) {
                      PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                          ((OrchestraManagementException)caught).getDetails());
                    } else {
                      PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while activating process !", caught.getMessage());
                    }
                  }
                });
          }
        }

        @Override
        public boolean isActive(final MProcess value) {
          return value.getProcessState().equals("retired");
        }
      };
    return activateAction;
  }

  private Cell<MProcess> getRetireActionCell() {
    final Cell<MProcess> retireAction =
      new ImageActionCell<MProcess>(ImageResources.INSTANCE.suspend(), ImageResources.INSTANCE.suspendDisabled(), "suspend", this.debugPrefix + "-suspend") {
        @Override
        public void execute(final MProcess object) {
          if (this.isActive(object)) {
            PListProcessCellTable.this.orchestramanagement.retireProcess(object.getProcessId(),
                new AsyncCallback<Void>() {
                  public void onSuccess(final Void arg0) {
                    ConsoleEventBus.getEventBus().fireEvent(new ViewRefreshEvent(object.getProcessId(), PTab.Type.LIST_PROCESS_ACTIVITY, true));
                  }
                  public void onFailure(final Throwable caught) {
                    GWT.log("PListProcess - retireProcess: on Failure : " + caught.getMessage());
                    if (caught instanceof OrchestraManagementException) {
                      PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                          ((OrchestraManagementException)caught).getDetails());
                    } else {
                      PPopup.launch(PPopup.PopupType.ErrorPopup, "Fail while retiring process !", caught.getMessage());
                    }
                  }
                });
          }
        }

        @Override
        public boolean isActive(final MProcess value) {
          return value.getProcessState().equals("active");
        }
      };
    return retireAction;
  }

  private Cell<MProcess> getProcessDetailsActionCell() {
    final Cell<MProcess> processDetailsAction = new ImageActionCell<MProcess>(ImageResources.INSTANCE.activities(),
        ImageResources.INSTANCE.activitiesDisabled(), "activities", this.debugPrefix + "-activities") {

      @Override
      public void execute(final MProcess object) {
        // create the presenter of list process activity
        new PListProcessActivity(PListProcessCellTable.this.tabLayoutPanel,
            object.getProcessId(),
            object.getProcessName());
      }

      @Override
      public boolean isActive(final MProcess value) {
        return true;
      }
    };
    return processDetailsAction;
  }

  private Cell<MProcess> getListInstancesActionCell() {
    final Cell<MProcess> listInstancesAction = new ImageActionCell<MProcess>(ImageResources.INSTANCE.instancesIcon(),
        ImageResources.INSTANCE.instancesIconDisabled(), "instances", this.debugPrefix + "-instances") {

      @Override
      public void execute(final MProcess object) {
        // create list instance presenter
        new PListInstance(PListProcessCellTable.this.tabLayoutPanel,
            object.getProcessId(), // and to finish, give the process Id
            object.getProcessName());
      }

      @Override
      public boolean isActive(final MProcess value) {
        return true;
      }
    };
    return listInstancesAction;
  }

  public void refresh() {
    if (this.processId == null) {
      // Add the data to the data provider, which automatically pushes it to the
      // widget.
      this.orchestramanagement.getProcesses(new AsyncCallback<List<MProcess>>() {

        public void onSuccess(final List<MProcess> result) {
          PListProcessCellTable.this.dataProvider.getList().clear();
          PListProcessCellTable.this.dataProvider.getList().addAll(result);
          ConsoleEventBus.getEventBus().fireEvent(new RefreshDoneEvent());
        }

        public void onFailure(final Throwable caught) {
          GWT.log("ProcessCellTable - refreshCurrentList: onFailure : " + caught.getMessage());
          ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
          if (caught instanceof OrchestraManagementException) {
            PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                ((OrchestraManagementException)caught).getDetails());
          } else {
            PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
          }
        }
      });
    } else {
      this.orchestramanagement.getProcess(this.processId, new AsyncCallback<MProcess>() {
        public void onSuccess(final MProcess process) {
          PListProcessCellTable.this.dataProvider.getList().clear();
          PListProcessCellTable.this.dataProvider.getList().add(process);
          ConsoleEventBus.getEventBus().fireEvent(new RefreshDoneEvent());
        }

        public void onFailure(final Throwable caught) {
          GWT.log("ProcessCellTable - refreshCurrentList: onFailure : " + caught.getMessage());
          ConsoleEventBus.getEventBus().fireEvent(new SuspendTimerEvent(true));
          if (caught instanceof OrchestraManagementException) {
            PPopup.launch(PPopup.PopupType.ErrorPopup, caught.getMessage(),
                ((OrchestraManagementException)caught).getDetails());
          } else {
            PPopup.launch(PPopup.PopupType.ErrorPopup, "Unable to access to Orchestra !", caught.getMessage());
          }
        }
      });
    }
  }

}
