/*******************************************************************************
 * Copyright (C) 2010  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 *******************************************************************************/
package org.ow2.orchestra.console.GWTConsole.client.ListProcessActivity;

import com.google.gwt.event.shared.HandlerRegistration;
import org.ow2.orchestra.console.GWTConsole.client.Commons.PresenterIF;
import org.ow2.orchestra.console.GWTConsole.client.ConsoleEventBus;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshEvent;
import org.ow2.orchestra.console.GWTConsole.client.Event.ViewRefreshHandler;
import org.ow2.orchestra.console.GWTConsole.client.ListProcess.PListProcessCellTable;
import org.ow2.orchestra.console.GWTConsole.client.ProcessDetails.PProcessViewerPanel;
import org.ow2.orchestra.console.GWTConsole.client.Tab.PTab;
import org.ow2.orchestra.console.GWTConsole.client.TabLayoutPanel.PTabLayoutPanel;

/**
 * Presenter of the process activity view
 *
 * @author Jean-Yves Goubet
 */
public class PListProcessActivity extends PresenterIF {

  private VListProcessActivityIF view;

  protected String processId = null;
  private ViewRefreshHandler refreshHandler = null;

  private HandlerRegistration handlerRegistration;

  private final PListProcessCellTable pProcessCellTable;

  private final PProcessViewerPanel processViewHandler;

  public PListProcessActivity(final PTabLayoutPanel pTabLayoutPanel, final String processId, final String processName) {
    //
    final VListProcessActivity vListProcessActivity = new VListProcessActivity();
    this.bindingView(vListProcessActivity);
    this.processId = processId;

    pTabLayoutPanel.addTab(processId, processName, PTab.Type.LIST_PROCESS_ACTIVITY, vListProcessActivity, this);
    //

    this.pProcessCellTable =
            new PListProcessCellTable(processId, pTabLayoutPanel, this.view.getProcessCellTable(),
                    "processDetailsCellTable-" + processId);

    this.view.getPager().setDisplay(this.view.getProcessActivitiesCellTable());
    this.view.getPager().ensureDebugId("processActivitiesPager-" + processId);
    this.addListeners();
    this.processViewHandler = new PProcessViewerPanel(processId, null, this.view.getViewerPanel(), pTabLayoutPanel);

    new PProcessActivityCellTable(processId, this.view.getProcessActivitiesCellTable(),
            "processActivitiesCellTable-" + processId, this.processViewHandler);
  }

  @Override
  protected void bindingView(final Object view) {
    this.view = (VListProcessActivityIF) view;
  }

  @Override
  protected void addListeners() {
    this.addViewRefreshHandlerToBus();
  }

  private void addViewRefreshHandlerToBus() {

    this.refreshHandler = new ViewRefreshHandler() {

      public void onRefresh(final ViewRefreshEvent viewRefreshEvent) {
        if (PListProcessActivity.this.processId.equals(viewRefreshEvent.getElementId())
            && viewRefreshEvent.getElementType() == PTab.Type.LIST_PROCESS_ACTIVITY) {
          PListProcessActivity.this.pProcessCellTable.refresh();
          PListProcessActivity.this.processViewHandler.refreshProcessView();
        }
      }
    };
    this.handlerRegistration = ConsoleEventBus.getEventBus().addHandler(ViewRefreshEvent.TYPE, this.refreshHandler);
  }

  @Override
  protected void setDebugID() {
    // no debug id
  }

  @Override
  public void destroy() {
    if (this.handlerRegistration != null) {
      this.handlerRegistration.removeHandler();
    }
  }

}
