/**
 * Copyright (C) 2011  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.GWTConsole.server;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Level;
import javax.xml.bind.JAXBException;
import org.granite.config.GraniteConfig;
import org.granite.context.SimpleGraniteContext;
import org.granite.messaging.amf.io.AMF3Serializer;
import org.ow2.orchestra.designer.bpmn.model.ProcessModel;
import org.ow2.orchestra.designer.models.exceptions.TransformationException;
import org.ow2.orchestra.designer.models.transformers.bpmn2designer.Bpmn2DesignerTransformer;
import org.ow2.orchestra.jaxb.bpmn.Definitions;
import org.ow2.orchestra.jaxb.bpmn.JaxbUtil;
import org.ow2.orchestra.util.Misc;
import org.xml.sax.SAXException;


/**
 * @author Guillaume Porcher
 */
public final class DesignerModelUtils {


  private DesignerModelUtils() {
  }

  private static GraniteConfig graniteConfig = null;

  public static byte[] getSerializedDesignerModel(final byte[] bpmnModel)
          throws JAXBException, SAXException, TransformationException, IOException {

    if (DesignerModelUtils.graniteConfig == null) {
      final InputStream inputStream = DesignerModelUtils.class.getResourceAsStream("granite-config.xml");
      if (inputStream == null) {
        Misc.slowLog(Level.SEVERE, "Granite config not found");
      }
      DesignerModelUtils.graniteConfig = new GraniteConfig(null, inputStream, null, null);
    }
    SimpleGraniteContext.createThreadIntance(DesignerModelUtils.graniteConfig, null, null);

    final Definitions definitions = JaxbUtil.unmarshalBpmn(bpmnModel);

    final ProcessModel processModel = Bpmn2DesignerTransformer.bpmnToModel(definitions);

    final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
    final AMF3Serializer amf3Serializer = new AMF3Serializer(outputStream, false);
    amf3Serializer.writeObject(processModel);
    amf3Serializer.flush();
    return outputStream.toByteArray();
  }
}
